% Bos_Knn.m
% This function performs K nearest neighbors classification on data using the function Knn.m
% It does a
% leave one out cross validation, and then tests performance against test
% data.  A confusion matrix is also displayed.
%
% It takes as arguments trainingData and testData.  A few things should be
% specified in this function (it's not well decomposed) -- this includes a
% vector listing all the classes in the data, and how many K's the cross
% validation should be performed on.  (Nk)
%
% This code was originally put together by Matt Adcock, and then modified
% by Jeevan Kalanithi
%
% matta@media.mit.edu
% jeevan@media.mit.edu


function BoS_KNN(trainData,testData)

training_data = trainData;
testing_data = testData;

% training_data = (W_jim_train*traind_n4')';
% testing_data = (W_jim_train*testd_n4')';

% training_data = (W_jim_total*data_n4')';
% testing_data = (W_jim_total*data_many_n4')';

% training_data = red_jim_train;
% testing_data = red_jim_test;



classes = [1 2 3 4 5]

NUM_CLASSES = length(classes);

Nk = 20

correct_training_trials = zeros(Nk,1);

for k = 1:Nk
    
    for i = 1:length(training_data)
       
        td = training_data;
        samp = td(i,2:end);
        true_class = td(i,1);
        td(i,:) = [];

        est_class = kNN ( td, samp, k, classes );
        %compare to 'true', and record as 'success'
        if (est_class == true_class);
            correct_training_trials(k) = correct_training_trials(k)+1;
        end
        
    end %i

end %k

%plot success results
percent_correct = correct_training_trials./length(training_data);
%plot(1:Nk,correct_training_trials(1:Nk),'o');
% hold on
plot(1:Nk,percent_correct(1:Nk),'+-');
xlabel('K');
ylabel('percent correct');
title('Knn classification for Single User, rational dim reduction Different Voting Schemes');
% axis([0 20 .87 .95]);

conf_mat = zeros(NUM_CLASSES,NUM_CLASSES);

max(percent_correct)

k = argmax(percent_correct)

for i = 1:length(testing_data)
    
    td = training_data;
    samp = testing_data(i,2:end);
	  true_class = testing_data(i,1);

	est_class = kNN ( td, samp, k, classes );
	%compare to 'true', and record as 'success'
    conf_mat(est_class,true_class) = conf_mat(est_class,true_class) +1;
        
end %i

conf_mat;
performance_rate = (sum(diag(conf_mat)))/length(testing_data);
performance_rate

bar3(conf_mat);
title('Knn confusion matrix, testing on All Users Data, Trained on Single User');

alpha(0.5);
xlabel('Actual Class');
ylabel('Estimated Class');
