function [ class ] = kNN( data, sample, k, classes )

% K nearest neighbors
%
% Computes the distance between a test sample and all other data points,
% chooses the k-nearest, and uses one of three voting schemes to pick a
% final winner.
%
% The distance calculations were written by matt adcock, the three voting
% schemes were implemented by jeevan kalanithi.  
%
% again, the decomp on the voting isn't brilliant; to choose a voting
% scheme, just comment out the three options at the bottom of this file.

%assume each training data sample is of the form [c x1 x2 x3 ... xd]
%where c is the actual class and d is the dim

%assume the sample to classify is in the form [x1 x2 x3 ... xd]

%k

NUM_CLASSES = 5;

data_classes = data(:,1);
data_values = data(:,2:end);

sz = size(data_values);
m = sz(1);

dists = sqrt(sum(abs(data_values - sample(ones(m,1),:)).^2 , 2));

d_c = [dists, data_classes];

sorted_results = sortrows(d_c,1);

% find the k nearest results

Knn = sorted_results(1:k,:);

% now we need to have a voting scheme that can break ties
winningCount = 0;
winners = zeros(1,NUM_CLASSES);
classByMeanDistance = ones(1,NUM_CLASSES).*max(Knn(:,1));

for i = 1:NUM_CLASSES
    Knn_this_class = Knn(Knn(:,2) == i,:);
    numPerClass(i) = length(Knn_this_class);
    % don't divide by zero...
    if (numPerClass(i) > 0) 
        classByMeanDistance(i) = mean(Knn_this_class(:,1));
    end
    % find the class who is the mode of the k nearest neighbors
    if (numPerClass(i) >= winningCount)
        % if numPerClass is strictly greater than the last winningCount,
        % there is only ONE winner so far, erase all other winners
        if (numPerClass(i) > winningCount)
            winners = zeros(1,NUM_CLASSES);
        end
        winners(i) = 1;
        winningCount = numPerClass(i);
    end
end

% is there one and only one winner...?
if (sum(winners) == 1)
    bestClass = argmax(winners);
% ...apparently not, we need to tie-break.  take the class with the
% smallest average distance
else
    bestClass = argmin(classByMeanDistance);
    % bestClass = 0;
end

if (k == 1)
    bestWeightedClass = argmax(winners);
else
    bestWeightedClass = argmin(classByMeanDistance); %weighted voting
end

class = bestWeightedClass; %weighted voting
% class = bestClass; % smarter tie-break voting
% class = mode(Knn(:,2)); % dumb voting



