% Bos_Parzen.m
% This function performs Parzen windows classification on data using the function myParzen.m
% It does a
% leave one out cross validation, and then tests performance against test
% data.  A confusion matrix is also displayed.
%
% It takes as arguments trainingData and testData.  A few things should be
% specified in this function (it's not well decomposed) -- this includes a
% vector listing all the classes in the data, and how many K's the volumes
% to be tried: startVolume, endVolume and volumeIncrement (volumeInc)
%
% This code was originally put together by Matt Adcock, and then modified
% by Jeevan Kalanithi
%
% matta@media.mit.edu
% jeevan@media.mit.edu

function BoS_Parzen(trainData,testData)

% training_data = red_jim_train;
% testing_data = red_jim_test;

training_data = trainData;
testing_data = testData;

% training_data = (W_jim_train*traind_n4')';
% testing_data = (W_jim_train*testd_n4')';

% training_data = (W_jim_total*data_n4')';
% testing_data = (W_jim_total*data_many_n4')';

classes = [1 2 3 4 5];
NUM_CLASSES = length(classes);


% do we add a class if we don't find anyone in our volume, or do we just
% guess a random known class?
PARZ_RANDOM_GUESS_ON = 1 ;

volumeEnd = 4; % number of different volumes to try
volumeStart = 0.1;
volumeInc = 0.1;

correct_training_trials = zeros(round(volumeEnd*10),1);
correct_test_trials = zeros(round(volumeEnd*10),1);


% try parzen for different volume sizes

for v = volumeStart:volumeInc:volumeEnd
    
    for i = 1:length(training_data)
       
        td = training_data;
        samp = td(i,2:end);
        true_class = td(i,1);
        td(i,:) = [];

        est_class = myParzen( td, samp, v, classes );
        %compare to 'true', and record as 'success'
        if (est_class == true_class);
            correct_training_trials(round(v*1/volumeInc)) = correct_training_trials(round(v*1/volumeInc))+1;
        end
        
    end %i

end %v
hold on

%plot success results
percent_correct = correct_training_trials./length(training_data);
%plot(1:volumeEnd,correct_training_trials(1:volumeEnd),'o');

% hold on
plot(volumeStart:volumeInc:volumeEnd,percent_correct,'m^');
xlabel('volume length');
ylabel('percent correct');
title('parzen classification for Single User, All data (leave-one-out), rational dim reduction');

conf_mat = zeros(NUM_CLASSES,NUM_CLASSES);

v = argmax(percent_correct)*volumeInc;

% for v = volumeStart:volumeInc:volumeEnd
for i = 1:length(testing_data)
    
    td = training_data;
    samp = testing_data(i,2:end);
	true_class = testing_data(i,1);

	est_class = myParzen( td, samp, v, classes );
    if (PARZ_RANDOM_GUESS_ON == 1)
        if (est_class > 5)
            est_class = round(random('uniform',1,NUM_CLASSES));
        end
    end
    
    if (est_class == true_class)
        correct_test_trials(round(v*10)) = correct_test_trials(round(v*1/volumeInc))+1;
    end

	% compare to 'true', and record as 'success'
    conf_mat(est_class,true_class) = conf_mat(est_class,true_class) +1;
        
end %i
%end %v

%plot success results
percent_test_correct = correct_test_trials./length(testing_data);
%plot(1:volumeEnd,correct_test_trials(1:volumeEnd),'o');
max(percent_correct)
% hold on
%plot(0.1:0.1:volumeEnd,percent_test_correct,'m^');
%xlabel('volume length');
%ylabel('percent correct');
%title('parzen classification for All User Data, rational dim reduction');


conf_mat;
performance_rate = (sum(diag(conf_mat)))/length(testing_data)
bar3(conf_mat);
title('parzen classification confusion matrix, testing on Single User Data, Trained on All Users, volume is 1.7');
alpha(0.5);
xlabel('Actual Class');
ylabel('Estimated Class');

