% Bos_ParzenClassFinder.m
%
% This script is meant to test whether we can use parzen window
% classification to detect when a new class enters the picture.
% We will do this by finding posteriors only P(x|wi), not by using parzen
% to find pdf's.  (ie, p(x) )
%
% The idea here is that, for a given volume size that we have found using
% the training data, can we start adding new samples to the training data
% of a new class? if these data do not seem to be falling into the other classes but
% do seem to be forming their own class, can we recognize this and create a
% new label "on the fly?"
%
% here is what we do: start with a set of labeled "training data."  do
% leave-one out validation on this dataset to determine general
% performance. this training data will contain data from all the classes
% except for one.  now do the real test:
%
% Start adding data from the test data into the training data.  Test each
% sample using the parzen method to try to determine its class. if parzen
% can find a label, associate the data with the label and see if you are
% right.  if the data cannot be associated with a label, mark it as class
% "c + 1."  Now, this method is very sensitive to the selection of volume
% size, since it's possible that two data points that are "really" from the
% same cluster will be classified as different labels if they are presented
% early and are sufficiently far from one another.
%
% we need to make sure that our removed class is relabeled as "c + 1," so
% if the classifier does generate a new class and it matches the removed
% class's data points, we will recognize this.  (keep in mind, if the
% classifier then makes some c + 2 points, this will not be classified as
% hits)
%
% written by jeevan kalanithi

novel_class = 5;

complete_class_list = [1 2 3 4 5];
training_class_list = setdiff(complete_class_list,novel_class);
new_class_list = complete_class_list(1:(length(complete_class_list)-1));


data = red_jim_total;
test_data = data(data(:,1) == novel_class,:); % make a data set of a "new, unknown" class (the novel_class)
% we should also sprinkle in samples from the other classes, but we are not
% yet doing so

% training data is from all the other classes
training_data = data(data(:,1) ~= novel_class,:);

test_data = relabelData(test_data,novel_class,complete_class_list(end));
training_data = relabelData(training_data,training_class_list,new_class_list);

% n1 = size(training1_data)
% n2 = size(training2_data)

%plot(training_data(:,1), training_data(:,2), 'r+')

classes = [1 2 3 4]

NUM_CLASSES = length(classes);

% volumeSize = 23; % best volume for accel data
startVolume = 0.1 % first volume size to check
volumeInc = 0.1
endVolume = 4 % last volume size to check

correct_training_trials = zeros(round(endVolume*(1/volumeInc)),1);
correct_test_trials = zeros(round(endVolume*(1/volumeInc)),1);


% try parzen for different volume sizes

% for v = startVolume:volumeInc:endVolume
%     
%     for i = 1:length(training_data)
%        
%         td = training_data;
%         samp = td(i,2:end);
%         true_class = td(i,1);
%         td(i,:) = [];
% 
%         est_class = myParzen( td, samp, v, classes );
%         
%         %compare to 'true', and record as 'success'
%         if (est_class == true_class);
%             correct_training_trials(round(v*(1/volumeInc))) = correct_training_trials(round(v*(1/volumeInc)))+1;
%         end
%         
%     end %i
% 
% end %v
% 
% %plot success results
% percent_correct_train = correct_training_trials./length(training_data);
%plot(1:endVolume,correct_training_trials(1:endVolume),'o');

% plot(startVolume:volumeInc:endVolume,percent_correct_train,'m^');
% xlabel('volume length');
% ylabel('percent correct');
% title('parzen classification for Single User, Training Data, Rational Reduced Data: Can we recognize a new class on the fly?');

for v = startVolume:volumeInc:endVolume
    td = training_data;
    for i = 1:length(test_data)

        %td = test_data;
        samp = test_data(i,2:end);
        true_class = test_data(i,1);
        %td(i,:) = [];

        est_class = myParzen( td, samp, v, classes );

        % our parzenator thinks we have a new class on our hands, add it to our
        % class list
        if (est_class == classes(end) + 1)
            classes = cat(2,classes,classes(end) + 1);
        end

        %compare to 'true', and record as 'success'
        if (est_class == true_class);
            correct_test_trials(round(v*(1/volumeInc))) = correct_test_trials(round(v*(1/volumeInc)))+1;
        end
        
        % tack on new sample to our increasing data set
        td = [td;test_data(i,:)];

    end %i
end %v

hold on

percent_correct_test = correct_test_trials./length(test_data);

plot(startVolume:volumeInc:endVolume,percent_correct_test,'md');
xlabel('volume length');
ylabel('percent correct');
title('Parzen for Single User, all rationally reduced data: Can we find new classes on the fly?');
%hold off

conf_mat = zeros(NUM_CLASSES,NUM_CLASSES);

% v = 4;
% 
% for i = 1:length(testing_data)
%     
%     td = training_data;
%     samp = testing_data(i,2:end);
% 	true_class = testing_data(i,1);
% 
% 	est_class = kNN ( td, samp, v, classes );
% 	%compare to 'true', and record as 'success'
%     conf_mat(est_class,true_class) = conf_mat(est_class,true_class) +1;
%         
% end %i
% 
% conf_mat
% performance_rate = (sum(diag(conf_mat))/length(testing_data)
% 
