function [ class ] = myParzen( data, sample, v, classes )
% Parzen Windows Classification
%
% Computes the distance between a test sample and all other data points,
% chooses those that fall within a certain distance, and uses one of three voting schemes to pick a
% final winner.
%
% The distance calculations were written by matt adcock, the three voting
% schemes and the sorting were implemented by jeevan kalanithi.  
%
% again, the decomp on the voting isn't brilliant; to choose a voting
% scheme, just comment out the three options at the bottom of this file.

%assume each training data sample is of the form [c x1 x2 x3 ... xd]
%where c is the actual class and d is the dim

%assume the sample to classify is in the form [x1 x2 x3 ... xd]

%v

thisDistance = v;%

NUM_CLASSES = length(classes);

data_classes = data(:,1);
data_values = data(:,2:end);

sz = size(data_values);
m = sz(1);

dists = sqrt(sum(abs(data_values - sample(ones(m,1),:)).^2 , 2));

d_c = [dists, data_classes];

sorted_results = sortrows(d_c,1);

% see which samples are in our volume

Knn = sorted_results(sorted_results(:,1) < thisDistance,:);

% nobody in our volume. make a new class, c + 1, and return that
if (length(Knn) == 0)
    class = classes(end) + 1;
    return;
end

% now we need to have a voting scheme that can break ties
winningCount = 0;
winners = zeros(1,NUM_CLASSES);
classByMeanDistance = ones(1,NUM_CLASSES).*max(Knn(:,1));

for i = 1:NUM_CLASSES
    Knn_this_class = Knn(Knn(:,2) == i,:);
    numPerClass(i) = length(Knn_this_class);
    % don't divide by zero...
    if (numPerClass(i) > 0) 
        classByMeanDistance(i) = mean(Knn_this_class(:,1));
    end
    % find the class who is the mode of the k nearest neighbors
    if (numPerClass(i) >= winningCount)
        % if numPerClass is strictly greater than the last winningCount,
        % there is only ONE winner so far, erase all other winners
        if (numPerClass(i) > winningCount)
            winners = zeros(1,NUM_CLASSES);
        end
        winners(i) = 1;
        winningCount = numPerClass(i);
    end
end

% is there one and only one winner...?
if (sum(winners) == 1)
    bestClass = argmax(winners);
% ...if no, we need to tie-break.  take the class with the
% smallest average distance
elseif (sum(winners) == 0)
    bestClass = 0; % nobody is in the volume
else    
    bestClass = argmin(classByMeanDistance);
end

class = argmin(classByMeanDistance); %weighted voting
% class = bestClass; %smarter tie-break voting
% class = mode(Knn(:,2)); %dumb voting



